#include <Wire.h>

#define MotorSpeedSet             0x82
#define PWMFrequenceSet           0x84
#define DirectionSet              0xaa
#define MotorSetA                 0xa1
#define MotorSetB                 0xa5
#define Nothing                   0x01

#define I2CMotorDriverAdd         0x0a   // Set the address of the I2CMotorDriver

const int ledPin = 13;

int ledState = 1;
int motorSpeed;
int motorState;

int speedA = 23;
int speedB = 25;

char incomingByte;
long prevMillis;

void setup() {
  pinMode(ledPin, OUTPUT);
  Wire.begin();
  Serial.begin(9600);
  MotorSpeedSetAB(speedA, speedB);
  Serial.print('1');
}

void loop() {
  long currentMillis = millis();
  if (Serial.available()) {
    incomingByte = Serial.read(); //read the serial data and store it in 'incomingByte'
  }
  switch (incomingByte) {         //using switch to check the incoming data and excuting the correct instructions
    case 'w':
      MotorDirectionSet(0b1010);
      prevMillis = currentMillis;
      break;
    case 'x':
      MotorDirectionSet(0b0101);
      prevMillis = currentMillis;
      break;
    case 'a':
      MotorDirectionSet(0b0110);
      prevMillis = currentMillis;
      break;
    case 'd':
      MotorDirectionSet(0b1001);
      prevMillis = currentMillis;
      break;
    case 's':
      motorSpeed = motorSpeed + 1;
      if (motorSpeed > 2) {
        motorSpeed = 0;
      }
      switch (motorSpeed) {                  //switch case to change speed and send UART signal to Edison
        case 0:
          speedA = 23;
          speedB = 25;
          MotorSpeedSetAB(speedA, speedB);
          Serial.print('1');
          break;
        case 1:
          speedA = 48;
          speedB = 55;
          MotorSpeedSetAB(speedA, speedB);
          Serial.print('2');
          break;
        case 2:
          speedA = 73;
          speedB = 80;
          MotorSpeedSetAB(speedA, speedB);
          Serial.print('3');
          break;
      }
      MotorSpeedSetAB(speedA, speedB);
      break;
    case 'l':                                 // case to power on/off the LED at pin 13
      if (ledState == 1) {
        digitalWrite(ledPin, ledState);
        ledState = 0;
      }
      else {
        digitalWrite(ledPin, ledState);
        ledState = 1;
      }
      break;
  }
  incomingByte = 0;
  if (currentMillis - prevMillis > 300) {   //stops the rover 300ms after moving
    MotorDirectionSet(0b0000);
  }
}


void MotorSpeedSetAB(unsigned char MotorSpeedA , unsigned char MotorSpeedB) {
  MotorSpeedA = map(MotorSpeedA, 0, 100, 0, 255);
  MotorSpeedB = map(MotorSpeedB, 0, 100, 0, 255);
  Wire.beginTransmission(I2CMotorDriverAdd); // transmit to device I2CMotorDriverAdd
  Wire.write(MotorSpeedSet);        // set pwm header
  Wire.write(MotorSpeedA);              // send pwma
  Wire.write(MotorSpeedB);              // send pwmb
  Wire.endTransmission();    // stop transmitting
}

//set the direction of DC motor.
void MotorDirectionSet(unsigned char Direction) {    //  Adjust the direction of the motors 0b0000 I4 I3 I2 I1
  Wire.beginTransmission(I2CMotorDriverAdd); // transmit to device I2CMotorDriverAdd
  Wire.write(DirectionSet);        // Direction control header
  Wire.write(Direction);              // send direction control information
  Wire.write(Nothing);              // need to send this byte as the third byte(no meaning)
  Wire.endTransmission();    // stop transmitting
}
